@@ -116,13 +116,11 @@ struct write_window {
 };
 
 static pthread_mutex_t window_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_rwlock_t window_fd_lock = PTHREAD_RWLOCK_INITIALIZER;
 static struct write_window window[NR_WINDOWS];
 
 static int
 evict_writes (int fd, uint64_t offset, size_t len)
 {
-  ACQUIRE_RDLOCK_FOR_CURRENT_SCOPE (&window_fd_lock);
   struct write_window oldest = { 0 };
 
   {
@@ -148,14 +146,10 @@ evict_writes (int fd, uint64_t offset, size_t len)
     window[NR_WINDOWS-1].offset = offset;
     window[NR_WINDOWS-1].len = len;
   }
-  /* Release window_lock here, but hold window_fd_lock since we don't
-   * want file_close -> remove_fd_from_window to close oldest.fd
-   */
+  /* Release lock here. */
 
   /* Evict the oldest window from the page cache (synchronously). */
   if (oldest.len > 0) {
-    int r;
-
     if (sync_file_range (oldest.fd, oldest.offset, oldest.len,
                          SYNC_FILE_RANGE_WAIT_BEFORE |
                          SYNC_FILE_RANGE_WRITE |
@@ -163,12 +157,9 @@ evict_writes (int fd, uint64_t offset, size_t len)
       nbdkit_error ("sync_file_range: cache=none: evicting oldest window: %m");
       return -1;
     }
-    r = posix_fadvise (oldest.fd, oldest.offset, oldest.len,
-                       POSIX_FADV_DONTNEED);
-    if (r != 0) {
-      errno = r;
+    if (posix_fadvise (oldest.fd, oldest.offset, oldest.len,
+                       POSIX_FADV_DONTNEED) == -1)
       nbdkit_debug ("posix_fadvise: POSIX_FADV_DONTNEED: (ignored): %m");
-    }
   }
 
   return 0;
@@ -180,7 +171,7 @@ evict_writes (int fd, uint64_t offset, size_t len)
 static void
 remove_fd_from_window (int fd)
 {
-  ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&window_fd_lock);
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
   size_t i;
 
   for (i = 0; i < NR_WINDOWS; ++i)
@@ -686,10 +677,8 @@ file_open (int readonly)
     /* This is a hint so we ignore failures. */
 #ifdef HAVE_POSIX_FADVISE
     int r = posix_fadvise (h->fd, 0, 0, fadvise_mode);
-    if (r != 0) {
-      errno = r;
-      nbdkit_debug ("posix_fadvise: %s: (ignored): %m", file);
-    }
+    if (r == -1)
+      nbdkit_debug ("posix_fadvise: %s: %m (ignored)", file);
 #else
     nbdkit_debug ("fadvise is not supported");
 #endif
@@ -1168,42 +1157,25 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 static int
 file_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
-  struct handle *h __attribute__ ((unused)) = handle;
-
 #ifdef FALLOC_FL_PUNCH_HOLE
-  if (h->can_punch_hole) {
-    int r;
+  struct handle *h = handle;
+  int r;
 
+  if (h->can_punch_hole) {
     r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
                       offset, count);
     if (r == -1) {
-      if (!is_enotsup (errno)) {
+      /* Trim is advisory; we don't care if it fails for anything other
+       * than EIO or EPERM. */
+      if (errno == EPERM || errno == EIO) {
         nbdkit_error ("fallocate: %m");
         return -1;
       }
 
-      h->can_punch_hole = false;
-    }
-  }
-#endif
+      if (is_enotsup (EOPNOTSUPP))
+        h->can_punch_hole = false;
 
-#ifdef BLKDISCARD
-  /* In future all Linux block devices may understand
-   * FALLOC_FL_PUNCH_HOLE which means this case would no longer be
-   * necessary, since the case above will handle it.
-   */
-  if (h->can_blkdiscard && IS_ALIGNED (offset | count, h->sector_size)) {
-    int r;
-    uint64_t range[2] = {offset, count};
-
-    r = ioctl (h->fd, BLKDISCARD, &range);
-    if (r == -1) {
-      if (!is_enotsup (errno)) {
-        nbdkit_error ("ioctl: BLKDISCARD: %m");
-        return -1;
-      }
-
-      h->can_blkdiscard = false;
+      nbdkit_debug ("ignoring failed fallocate during trim: %m");
     }
   }
 #endif
@@ -1313,7 +1285,7 @@ file_cache (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   struct handle *h = handle;
   int r;
 
-  /* This function returns an errno. */
+  /* Cache is advisory, we don't care if this fails */
   r = posix_fadvise (h->fd, offset, count, POSIX_FADV_WILLNEED);
   if (r) {
     errno = r;
