@@ -91,7 +91,7 @@ print_gnutls_error (int err, const char *fs, ...)
 }
 
 /* Try to load certificates from 'path'.  Returns true if successful.
- * If it's not a certicate directory it returns false.  Exits on
+ * If it's not a certificate directory it returns false.  Exits on
  * other errors.
  */
 static int
@@ -809,18 +809,20 @@ get_peer_dn (const char *fn, get_dn3_fn get_dn3)
   char *ret = NULL;
 
   if (!session) {
-    debug ("nbdkit_peer_tls_dn: no TLS session");
+    nbdkit_debug ("nbdkit_peer_tls_dn: no TLS session");
     goto out_no_dn;
   }
 
   cred = gnutls_auth_get_type (session);
   if (cred != GNUTLS_CRD_CERTIFICATE) {
-    debug ("nbdkit_peer_tls_dn: TLS session not using certificates");
+    nbdkit_debug ("nbdkit_peer_tls_dn: "
+                  "TLS session not using certificates");
     goto out_no_dn;
   }
 
   if (gnutls_certificate_type_get (session) != GNUTLS_CRT_X509) {
-    debug ("nbdkit_peer_tls_dn: TLS session not using X.509 certificates");
+    nbdkit_debug ("nbdkit_peer_tls_dn: "
+                  "TLS session not using X.509 certificates");
     goto out_no_dn;
   }
 
@@ -830,8 +832,8 @@ get_peer_dn (const char *fn, get_dn3_fn get_dn3)
      * happen because gnutls cannot allocate the certificate list but
      * that should be a rare error.  Return no DN in this case.
      */
-    debug ("nbdkit_peer_tls_dn: "
-           "no client certificates (is --tls-verify-peer set?)");
+    nbdkit_debug ("nbdkit_peer_tls_dn: "
+                  "no client certificates (is --tls-verify-peer set?)");
     goto out_no_dn;
   }
 
