@@ -33,6 +33,7 @@
 #ifndef NBDKIT_INTERNAL_H
 #define NBDKIT_INTERNAL_H
 
+#include <stdio.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdarg.h>
@@ -42,6 +43,27 @@
 #include <sys/socket.h>
 #endif
 
+#ifdef ENABLE_PROBES
+#include <sys/sdt.h>
+#else /* !ENABLE_PROBES */
+/* If probes are disabled, allow the macros to appear but they don't
+ * expand to anything.
+ */
+#define DTRACE_PROBE(pv,pb)
+#define DTRACE_PROBE1(pv,pb,a1)
+#define DTRACE_PROBE2(pv,pb,a1,a2)
+#define DTRACE_PROBE3(pv,pb,a1,a2,a3)
+#define DTRACE_PROBE4(pv,pb,a1,a2,a3,a4)
+#define DTRACE_PROBE5(pv,pb,a1,a2,a3,a4,a5)
+#define DTRACE_PROBE6(pv,pb,a1,a2,a3,a4,a5,a6)
+#define DTRACE_PROBE7(pv,pb,a1,a2,a3,a4,a5,a6,a7)
+#define DTRACE_PROBE8(pv,pb,a1,a2,a3,a4,a5,a6,a7,a8)
+#define DTRACE_PROBE9(pv,pb,a1,a2,a3,a4,a5,a6,a7,a8,a9)
+#define DTRACE_PROBE10(pv,pb,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
+#define DTRACE_PROBE11(pv,pb,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11)
+#define DTRACE_PROBE12(pv,pb,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12)
+#endif /* !ENABLE_PROBES */
+
 /* Do we have the --timeout option?
  *
  * OpenBSD defines timer_create as a function that returns -ENOSYS,
@@ -123,6 +145,7 @@ enum log_to {
   LOG_TO_STDERR,         /* --log=stderr forced on the command line */
   LOG_TO_SYSLOG,         /* --log=syslog forced on the command line */
   LOG_TO_NULL,           /* --log=null forced on the command line */
+  LOG_TO_FILE,           /* --log=/path used on the command line */
 };
 
 enum service_mode {
@@ -143,6 +166,8 @@ extern bool foreground;
 extern const char *ipaddr;
 extern bool keepalive;
 extern enum log_to log_to;
+extern const char *log_to_file;
+extern FILE *log_to_fp;
 extern unsigned mask_handshake;
 extern bool newstyle;
 extern bool no_mc;
@@ -183,8 +208,6 @@ extern struct backend *top;
 extern void debug_in_server (const char *msg, ...)
   ATTRIBUTE_FORMAT_PRINTF (1, 2);
 
-#define nbdkit_debug _Pragma("GCC error \"Use debug() instead of nbdkit_debug() inside the server\"")
-
 /* quit.c */
 extern volatile int quit;
 #ifndef WIN32
@@ -369,8 +392,9 @@ extern void free_debug_flags (void);
 extern void log_verror (const char *fs, va_list args);
 
 /* log-*.c */
-extern void log_stderr_verror (int orig_errno, const char *fs, va_list args)
-  ATTRIBUTE_FORMAT_PRINTF (2, 0);
+extern void log_fp_verror (FILE *fp,
+                           int orig_errno, const char *fs, va_list args)
+  ATTRIBUTE_FORMAT_PRINTF (3, 0);
 extern void log_syslog_verror (int orig_errno, const char *fs, va_list args)
   ATTRIBUTE_FORMAT_PRINTF (2, 0);
 
