@@ -81,15 +81,10 @@
 #endif
 
 static char *make_random_fifo (void);
-static void cleanup_random_fifo (void);
 static struct backend *open_plugin_so (size_t i, const char *filename,
                                        int short_name);
 static struct backend *open_filter_so (struct backend *next, size_t i,
                                        const char *filename, int short_name);
-static void parse_key_values (int argc, char **argv, bool is_first,
-                              const char *magic_config_key);
-static void parse_key_value (const char *arg,
-                             bool is_first, const char *magic_config_key);
 static void start_serving (void);
 static void write_pidfile (void);
 static bool is_config_key (const char *key, size_t len);
@@ -105,6 +100,8 @@ bool foreground;                /* -f */
 const char *ipaddr;             /* -i */
 bool keepalive;                 /* --keepalive */
 enum log_to log_to = LOG_TO_DEFAULT; /* --log */
+const char *log_to_file;        /* --log=/path */
+FILE *log_to_fp;                /* --log=/path */
 unsigned mask_handshake = ~0U;  /* --mask-handshake */
 bool newstyle = true;           /* false = -o, true = -n */
 bool no_mc;                     /* --no-meta-contexts */
@@ -269,7 +266,7 @@ dump_environment (void)
   size_t i;
 
   for (i = 0; environ[i]; ++i)
-    debug ("%s", environ[i]);
+    nbdkit_debug ("%s", environ[i]);
 }
 
 int
@@ -280,11 +277,13 @@ main (int argc, char *argv[])
   int tls_set_on_cli = false;
   bool short_name;
   const char *filename;
+  char *p;
   static struct filter_filename {
     struct filter_filename *next;
     const char *filename;
   } *filter_filenames = NULL;
   size_t i;
+  const char *magic_config_key;
 
   error_if_stdio_closed ();
   winsock_init ();
@@ -319,7 +318,6 @@ main (int argc, char *argv[])
     switch (c) {
     case DUMP_CONFIG_OPTION:
       dump_config ();
-      cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
 
     case DUMP_PLUGIN_OPTION:
@@ -344,12 +342,6 @@ main (int argc, char *argv[])
       {
         struct filter_filename *t;
 
-        if (optarg[0] == '\0') {
-          fprintf (stderr, "%s: filter name cannot be an empty string\n",
-                   program_name);
-          exit (EXIT_FAILURE);
-        }
-
         t = malloc (sizeof *t);
         if (t == NULL) {
           perror ("malloc");
@@ -370,15 +362,22 @@ main (int argc, char *argv[])
       break;
 
     case LOG_OPTION:
-      if (strcmp (optarg, "stderr") == 0)
+      if (strcmp (optarg, "default") == 0)
+        log_to = LOG_TO_DEFAULT;
+      else if (strcmp (optarg, "stderr") == 0)
         log_to = LOG_TO_STDERR;
       else if (strcmp (optarg, "syslog") == 0)
         log_to = LOG_TO_SYSLOG;
       else if (strcmp (optarg, "null") == 0)
         log_to = LOG_TO_NULL;
+      else if (optarg[0] == '/') {
+        log_to = LOG_TO_FILE;
+        log_to_file = optarg;
+      }
       else {
         fprintf (stderr, "%s: "
-                 "--log must be \"stderr\", \"syslog\" or \"null\"\n",
+                 "--log must be \"stderr\", \"syslog\", \"null\", or "
+                 "an absolute path\n",
                  program_name);
         exit (EXIT_FAILURE);
       }
@@ -390,7 +389,6 @@ main (int argc, char *argv[])
             strcmp (long_options[i].name, "short-options") != 0)
           printf ("--%s\n", long_options[i].name);
       }
-      cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
 
     case MASK_HANDSHAKE_OPTION:
@@ -430,7 +428,6 @@ main (int argc, char *argv[])
         if (short_options[i] != ':')
           printf ("-%c\n", short_options[i]);
       }
-      cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
 
     case SWAP_OPTION:
@@ -566,7 +563,7 @@ main (int argc, char *argv[])
     case 't':
       if (nbdkit_parse_unsigned ("threads", optarg, &threads) == -1)
         exit (EXIT_FAILURE);
-      /* XXX Worth a maximimum limit on threads? */
+      /* XXX Worth a maximum limit on threads? */
       break;
 
     case 'U':
@@ -605,12 +602,10 @@ main (int argc, char *argv[])
   if (optind >= argc) {
     if (help) {
       usage ();
-      cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
     }
     if (version) {
       display_version ();
-      cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
     }
     if (dump_plugin) {
@@ -620,7 +615,6 @@ main (int argc, char *argv[])
                "'nbdkit /path/to/plugin." SOEXT " --dump-plugin' or\n"
                "if you want to find out about the server use --dump-config\n",
                program_name);
-      cleanup_random_fifo ();
       exit (EXIT_FAILURE);
     }
 
@@ -647,9 +641,26 @@ main (int argc, char *argv[])
    */
   umask (0022);
 
-  /* If we will or might use syslog. */
-  if (log_to == LOG_TO_SYSLOG || log_to == LOG_TO_DEFAULT)
+  /* Open the log file. */
+  switch (log_to) {
+    /* If we will or might use syslog. */
+  case LOG_TO_DEFAULT:
+  case LOG_TO_SYSLOG:
     openlog (program_name, LOG_PID, 0);
+    break;
+
+  case LOG_TO_FILE:
+    log_to_fp = fopen (log_to_file, "a");
+    if (log_to_fp == NULL) {
+      fprintf (stderr, "%s: %s: fopen: %m\n", program_name, log_to_file);
+      exit (EXIT_FAILURE);
+    }
+    break;
+
+  case LOG_TO_STDERR:
+  case LOG_TO_NULL:
+    ;
+  }
 
   /* Print the version in debug output, right after syslog initialization. */
   if (strcmp (NBDKIT_VERSION_EXTRA, "") == 0)
@@ -724,11 +735,6 @@ main (int argc, char *argv[])
    * help/version/plugin information.
    */
   filename = argv[optind++];
-  if (filename[0] == '\0') {
-    fprintf (stderr, "%s: plugin name cannot be an empty string\n",
-             program_name);
-    exit (EXIT_FAILURE);
-  }
   short_name = is_short_name (filename);
 
   /* Is there an executable script located in the plugindir?
@@ -799,7 +805,6 @@ main (int argc, char *argv[])
       b->usage (b);
     }
     top->free (top);
-    cleanup_random_fifo ();
     exit (EXIT_SUCCESS);
   }
 
@@ -815,12 +820,50 @@ main (int argc, char *argv[])
       printf ("\n");
     }
     top->free (top);
-    cleanup_random_fifo ();
     exit (EXIT_SUCCESS);
   }
 
-  /* Parse key=value on the command line. */
-  parse_key_values (argc, argv, true, top->magic_config_key (top));
+  /* Call config and config_complete to parse the parameters.
+   *
+   * If the plugin provides magic_config_key then any "bare" values
+   * (ones not containing "=") are prefixed with this key.
+   *
+   * For backwards compatibility with old plugins, and to support
+   * scripting languages, if magic_config_key == NULL then if the
+   * first parameter is bare it is prefixed with the key "script", and
+   * any other bare parameters are errors.
+   *
+   * Keys must live for the life of nbdkit.  Since we want to avoid
+   * modifying argv (so that /proc/PID/cmdline remains sane) but we
+   * need to create a key from argv[i] = "key=value" we must intern
+   * the keys, which are then freed at the end of main().
+   */
+  magic_config_key = top->magic_config_key (top);
+  for (i = 0; optind < argc; ++i, ++optind) {
+    size_t n;
+
+    p = strchr (argv[optind], '=');
+    n = p - argv[optind];
+    if (p && is_config_key (argv[optind], n)) { /* Is it key=value? */
+      const char *key = nbdkit_strndup_intern (argv[optind], n);
+      if (key == NULL)
+        exit (EXIT_FAILURE);
+      top->config (top, key, p+1);
+    }
+    else if (magic_config_key == NULL) {
+      if (i == 0)               /* magic script parameter */
+        top->config (top, "script", argv[optind]);
+      else {
+        fprintf (stderr,
+                 "%s: expecting key=value on the command line but got: %s\n",
+                 program_name, argv[optind]);
+        exit (EXIT_FAILURE);
+      }
+    }
+    else {                      /* magic config key */
+      top->config (top, magic_config_key, argv[optind]);
+    }
+  }
 
   /* This must run after parsing the parameters so that the script can
    * be loaded for scripting languages.  But it must be called before
@@ -831,7 +874,6 @@ main (int argc, char *argv[])
     top->dump_fields (top);
     top->free (top);
     free_interns ();
-    cleanup_random_fifo ();
     exit (EXIT_SUCCESS);
   }
 
@@ -874,7 +916,16 @@ main (int argc, char *argv[])
   free (pidfile);
   free (uri);
 
-  cleanup_random_fifo ();
+  if (random_fifo) {
+    unlink (random_fifo);
+    free (random_fifo);
+  }
+
+  if (random_fifo_dir) {
+    rmdir (random_fifo_dir);
+    free (random_fifo_dir);
+  }
+
   crypto_free ();
   close_quit_pipe ();
 
@@ -913,24 +964,14 @@ make_random_fifo (void)
   return sock;
 
  error:
-  cleanup_random_fifo ();
+  free (random_fifo);
+  random_fifo = NULL;
+  rmdir (random_fifo_dir);
+  free (random_fifo_dir);
+  random_fifo_dir = NULL;
   return NULL;
 }
 
-static void
-cleanup_random_fifo (void)
-{
-  if (random_fifo) {
-    unlink (random_fifo);
-    free (random_fifo);
-  }
-
-  if (random_fifo_dir) {
-    rmdir (random_fifo_dir);
-    free (random_fifo_dir);
-  }
-}
-
 const char *
 service_mode_string (enum service_mode sm)
 {
@@ -1097,68 +1138,6 @@ open_filter_so (struct backend *next, size_t i,
   return ret;
 }
 
-/* Call config() on each of the key=value plugin & filter parameters. */
-static void
-parse_key_values (int argc, char **argv, bool is_first,
-                  const char *magic_config_key)
-{
-  if (optind >= argc) return;
-
-  parse_key_value (argv[optind], is_first, magic_config_key);
-
-  ++optind;
-#if defined(__GNUC__) && __GNUC__ >= 15 && defined(__OPTIMIZE__)
-  /* Ensure this is optimized as a tail call. */
-  __attribute__((musttail)) return
-#endif
-  parse_key_values (argc, argv, false, magic_config_key);
-}
-
-/* Parse a single [key=]value, calling config().
- *
- * If the plugin provides magic_config_key then any "bare" values
- * (ones not containing "=") are prefixed with this key.
- *
- * For backwards compatibility with old plugins, and to support
- * scripting languages, if magic_config_key == NULL then if the
- * first parameter is bare it is prefixed with the key "script", and
- * any other bare parameters are errors.
- *
- * Keys must live for the life of nbdkit.  Since we want to avoid
- * modifying argv (so that /proc/PID/cmdline remains sane) but we
- * need to create a key from argv[i] = "key=value" we must intern
- * the keys, which are then freed at the end of main().
- */
-static void
-parse_key_value (const char *arg,
-                 bool is_first, const char *magic_config_key)
-{
-  const char *p, *key;
-  size_t n;
-
-  p = strchr (arg, '=');
-  n = p - arg;
-  if (p && is_config_key (arg, n)) { /* Is it key=value? */
-    key = nbdkit_strndup_intern (arg, n);
-    if (key == NULL)
-      exit (EXIT_FAILURE);
-    top->config (top, key, p+1);
-  }
-  else if (magic_config_key == NULL) {
-    if (is_first)               /* magic script parameter */
-      top->config (top, "script", arg);
-    else {
-      fprintf (stderr,
-               "%s: expecting key=value on the command line but got: %s\n",
-               program_name, arg);
-      exit (EXIT_FAILURE);
-    }
-  }
-  else {                        /* magic config key */
-    top->config (top, magic_config_key, arg);
-  }
-}
-
 static void
 start_serving (void)
 {
